@@ -1,4 +1,4 @@
-package Mojolicious::Plugin::Authentication::OIDC 0.02;
+package Mojolicious::Plugin::Authentication::OIDC 0.03;
 use v5.26;
 use warnings;
 
@@ -20,10 +20,10 @@ integrated into Mojolicious
   });
 
   # in controller
-  say "Hi " . $c->current_user->firstname;
+  say "Hi " . $c->authn->current_user->firstname;
 
   use Array::Utils qw(intersect);
-  if(intersect($c->current_user_roles->@*, qw(admin))) { ... }
+  if(intersect($c->authn->current_user_roles->@*, qw(admin))) { ... }
 
 =head1 DESCRIPTION
 
@@ -73,7 +73,7 @@ Readonly::Hash my %DEFAULT_PARAMS => (
   redirect_path => '/auth',
   make_routes   => 1,
 
-  on_success => sub ($c, $token) {$c->session(token => $token); $c->redirect_to('/login/success')},
+  on_success => sub ($c, $token, $url) {$c->session(token => $token); $c->redirect_to($url)},
   on_error   => sub ($c, $error) {$c->render(json => $error)},
 
   get_token => sub ($c) {$c->session('token')},
@@ -87,6 +87,7 @@ Readonly::Hash my %DEFAULT_CONSTANTS => (
   response_type => 'code',
   grant_type    => 'authorization_code',
 );
+Readonly::Scalar my $DEFAULT_PREFIX => 'authn';
 
 =head1 METHODS
 
@@ -252,6 +253,13 @@ Otherwise, returns C<undef>
 =cut
 
 sub register($self, $app, $params) {
+  # Prefix handling
+  my $prefix = $params->{prefix} // $DEFAULT_PREFIX;
+  $prefix .= '.' if ($prefix);
+  my $params_helper             = "__oidc_params";
+  my $token_helper              = "__oidc_token";
+  my $current_user_helper       = "${prefix}current_user";
+  my $current_user_roles_helper = "${prefix}current_user_roles";
   # Parameter handling
   my %conf = (%DEFAULT_CONSTANTS, %DEFAULT_PARAMS, client_id => lc($app->moniker));
   $conf{$_} = $params->{$_} foreach (grep {exists($params->{$_})} (keys(%DEFAULT_PARAMS), @REQUIRED_PARAMS, @ALLOWED_PARAMS));
@@ -261,11 +269,11 @@ sub register($self, $app, $params) {
 
   # wrap success handler so that we can call login handler before finishing the req
   my $success_handler = $conf{on_success};
-  $conf{on_success} = sub($c, $token) {
-    my $token_data = $c->_oidc_token($token);
+  $conf{on_success} = sub($c, $token, $url) {
+    my $token_data = $c->app->renderer->get_helper($token_helper)->($c, $token);
     my $user       = $conf{get_user}->($token_data);
     $conf{on_login}->($c, $user) if ($conf{on_login});
-    return $success_handler->($c, $token);
+    return $success_handler->($c, $token, $url);
   };
 
   # Add our controller to the namespace for calling via routes or, e.g., OpenAPI
@@ -274,36 +282,40 @@ sub register($self, $app, $params) {
   # Fetch actual endpoints from well-known URL
   my $resp = Mojo::UserAgent->new()->get($conf{well_known_url});
   die("Unable to determine OIDC endpoints (" . $resp->res->error->{message} . ")\n") if ($resp->res->is_error);
-  @conf{qw(auth_endpoint token_endpoint)} = @{$resp->res->json}{qw(authorization_endpoint token_endpoint)};
+  @conf{qw(auth_endpoint token_endpoint logout_endpoint)} =
+    @{$resp->res->json}{qw(authorization_endpoint token_endpoint end_session_endpoint)};
 
   # internal helper for stored parameters (only to be used by OpenIDConnect controller)
   $app->helper(
-    _oidc_params => sub {
+    $params_helper => sub {
       return {map {$_ => $conf{$_}}
-          qw(auth_endpoint scope response_type login_path token_endpoint client_id client_secret grant_type on_error on_success)};
+          qw(auth_endpoint scope response_type login_path token_endpoint client_id client_secret grant_type on_error on_success logout_endpoint)
+      };
     }
   );
 
   # internal helper for decoded auth token. Pass the token in, or it'll be retrieved
   # via `get_token` handler
   $app->helper(
-    _oidc_token => sub($c, $token = undef) {
+    $token_helper => sub($c, $token = undef, $decode = 1) {
+      my $t = $token // $conf{get_token}->($c);
+      return $t unless ($decode);
       return decode_jwt(token => ($token // $conf{get_token}->($c)), key => \$conf{public_key});
     }
   );
 
   # public helper to access current user and OIDC roles
   $app->helper(
-    current_user => sub($c) {
-      return $conf{get_user}->($c->_oidc_token);
+    $current_user_helper => sub($c) {
+      return $conf{get_user}->($c->app->renderer->get_helper($token_helper)->($c));
     }
   );
   $app->helper(
-    current_user_roles => sub($c) {
+    $current_user_roles_helper => sub($c) {
       my ($user, $token);
       try {
-        $token = $c->_oidc_token;
-        $user  = $c->current_user;
+        $token = $c->app->renderer->get_helper($token_helper)->($c);
+        $user  = $c->renderer($current_user_helper)->();
         my @roles = $conf{get_roles}->($user, $token)->@*;
         @roles = grep {defined} map {$conf{role_map}->{$_}} @roles if (defined($conf{role_map}));
         return [@roles];
@@ -318,7 +330,7 @@ sub register($self, $app, $params) {
   $app->hook(
     before_dispatch => sub($c) {
       my $u;
-      try {$u = $c->current_user;} catch ($e) {
+      try {$u = $c->renderer($current_user_helper)->();} catch ($e) {
       }
       $conf{on_activity}->($c, $u) if ($u);
     }
